@@ -1,8 +1,8 @@
 #!/bin/sh
 # the next line restarts using wish \
-exec wish8.2 "$0" "$@"
+exec wish8.3 "$0" "$@"
 
-package require -exact snack 2.0
+package require -exact snack 2.2
 
 set width 400
 set height 100
@@ -48,8 +48,8 @@ pack [ button .f3.b2 -text ex2.wav -command {s read ex2.wav}] -side left
 proc Record {} {
     .c itemconf wave -pixelspersecond 300 -width 300
     s record
-    after cancel {.f3.bs invoke}
-    after 10000 {.f3.bs invoke}
+    after cancel [list catch {.f3.bs invoke}]
+    after 10000  [list catch {.f3.bs invoke}]
 }
 
 pack [ frame .f4] -pady 2
