@@ -19,7 +19,7 @@
 # Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 #
 
-package provide snack 2.0
+package provide snack 2.2
 
 # Set playback latency according to the environment variable PLAYLATENCY
 
@@ -112,7 +112,7 @@ namespace eval snack {
 	if {[snack::mixer outputs] != ""} {
 	    pack [label $wi.f.f2.lo -text "Output jacks:"]
 	    foreach jack [snack::mixer outputs] {
-		snack::mixer lines $jack [namespace current]::v(out$jack)
+		snack::mixer output $jack [namespace current]::v(out$jack)
 		pack [checkbutton $wi.f.f2.b$jack -text $jack \
 			-variable [namespace current]::v(out$jack)] \
 			-anchor w
@@ -132,6 +132,7 @@ namespace eval snack {
 	    {-title       "" "" "Open file"}
 	    {-initialdir  "" "" "."}
 	    {-initialfile "" "" ""}
+	    {-multiple    "" "" 0}
 	    {-format      "" "" "none"}
 	}
 	
@@ -151,7 +152,29 @@ namespace eval snack {
 	if {$data(-initialdir) == ""} {
 	    set data(-initialdir) "."
 	}
-	return [tk_getOpenFile -title $data(-title) -filetypes [loadTypes $data(-format)] -defaultextension [fmt2ext $data(-format)] -initialdir $data(-initialdir) -initialfile $data(-initialfile)]
+        if {[string match Darwin $::tcl_platform(os)]} {
+	 return [tk_getOpenFile -title $data(-title) \
+		    -multiple $data(-multiple) \
+		    -filetypes [loadTypes $data(-format)] \
+		    -defaultextension [fmt2ext $data(-format)] \
+		     -initialdir $data(-initialdir)]
+	}
+	# Later Tcl's allow multiple files returned as a list
+	if {$::tcl_version <= 8.3} {
+	    set res [tk_getOpenFile -title $data(-title) \
+		    -filetypes [loadTypes $data(-format)] \
+		    -defaultextension [fmt2ext $data(-format)] \
+		    -initialdir $data(-initialdir) \
+		    -initialfile $data(-initialfile)]
+	} else {
+	    set res [tk_getOpenFile -title $data(-title) \
+		    -multiple $data(-multiple) \
+		    -filetypes [loadTypes $data(-format)] \
+		    -defaultextension [fmt2ext $data(-format)] \
+		    -initialdir $data(-initialdir) \
+		    -initialfile $data(-initialfile)]
+	}
+	return $res
     }
 
     set loadTypes ""
@@ -251,7 +274,19 @@ namespace eval snack {
 	if {$data(-initialdir) == ""} {
 	    set data(-initialdir) "."
 	}
-	return [tk_getSaveFile -title $data(-title) -filetypes [saveTypes $data(-format)] -defaultextension [fmt2ext $data(-format)] -initialdir $data(-initialdir) -initialfile $data(-initialfile)]
+	if {[string match macintosh $::tcl_platform(platform)]} {
+	  set tmp [tk_getSaveFile -title $data(-title) \
+	      -initialdir $data(-initialdir) -initialfile $data(-initialfile)]
+	  if {[string compare [file ext $tmp] ""] == 0} {
+	    append tmp [fmt2ext $data(-format)]
+	  }
+	  return $tmp
+	} else {
+	  return [tk_getSaveFile -title $data(-title) \
+	      -filetypes [saveTypes $data(-format)] \
+	      -defaultextension [fmt2ext $data(-format)] \
+	      -initialdir $data(-initialdir) -initialfile $data(-initialfile)]
+	}
     }
 
     set saveTypes ""
@@ -452,6 +487,9 @@ namespace eval snack {
 		-fill black \
 		-draw0 0
 	]
+        if {[string match unix $::tcl_platform(platform)] } {
+	 set a(-font) {Helvetica 10}
+	}
 	array set a $args
 
 	if {$height <= 0} return
@@ -523,16 +561,37 @@ namespace eval snack {
 		-font {Helvetica 8} \
 		-starttime 0.0 \
 		-fill black \
+		-format time \
 		-draw0 0 \
 		-drawvisible 0
 	]
+        if {[string match unix $::tcl_platform(platform)] } {
+	 set a(-font) {Helvetica 10}
+	}
 	array set a $args
 
 	if {$pps <= 0.004} return
 
-	set deltalist [list .0001 .0002 .0005 .001 .002 .005 \
-		.01 .02 .05 .1 .2 .5 1 2 5 \
-		10 20 30 60 120 240 360 600 900 1800 3600 7200 14400]
+        switch -- $a(-format) {
+	 time {
+	  set deltalist [list .0001 .0002 .0005 .001 .002 .005 \
+			     .01 .02 .05 .1 .2 .5 1 2 5 \
+			     10 20 30 60 120 240 360 600 900 1800 3600 7200 14400]
+	 }
+	 "PAL frames" {
+	  set deltalist [list .04 .08 .4 .8 2 4 \
+			     10 20 50 100 200 500 1000 2000 5000 10000 20000]
+	 }
+	 "NTSC frames" {
+	  set deltalist [list .03333333333334 .0666666666667 \
+			     .3333333333334 .666666666667 1 2 4 \
+			     10 20 50 100 200 500 1000 2000 5000 10000 20000]
+	 }
+	 "10ms frames" {
+	  set deltalist [list .01 .02 .05 .1 .2 .5 1 2 5 \
+			     10 20 50 100 200 500 1000 2000 5000 10000 20000]
+	 }
+	}
 
 	set majTickH [expr {$height - [font metrics $a(-font) -linespace]}]
 	set minTickH [expr {$majTickH / 2}]
@@ -588,7 +647,7 @@ namespace eval snack {
 	    set ft 0
 	    set fx 0.0
 	}
-
+	
 	set lx [expr {($ox + $width) * [lindex [$canvas xview] 0] - 50}]
 	set rx [expr {($ox + $width) * [lindex [$canvas xview] 1] + 50}]
 
@@ -605,24 +664,36 @@ namespace eval snack {
 	    if {$a(-drawvisible) && $x < $lx} continue
 	    if {$a(-drawvisible) && $x > $rx} break
 
-	    set t [expr {$j * $dt + $ft}]
-
-	    if {$maxtime < 60} {
-		set tmp [expr {int($t)}]
-	    } elseif {$maxtime < 3600} {
-		set tmp x[clock format [expr {int($t)}] -format "%M:%S" -gmt 1]
-		regsub x0 $tmp "" tmp
-		regsub x $tmp "" tmp
-	    } else {
-		set tmp [clock format [expr {int($t)}] -format "%H:%M:%S" -gmt 1]
-	    }	    
-	    if {$dt < 1.0} {
-		set t $tmp[string trimleft [format "%.${ndec}f" \
-			[expr {($t-int($t))}]] 0]
-	    } else {
-		set t $tmp
+	    switch -- $a(-format) {
+	     time {
+	      set t [expr {$j * $dt + $ft}]
+	      
+	      if {$maxtime < 60} {
+	       set tmp [expr {int($t)}]
+	      } elseif {$maxtime < 3600} {
+	       set tmp x[clock format [expr {int($t)}] -format "%M:%S" -gmt 1]
+	       regsub x0 $tmp "" tmp
+	       regsub x $tmp "" tmp
+	      } else {
+	       set tmp [clock format [expr {int($t)}] -format "%H:%M:%S" -gmt 1]
+	      }	    
+	      if {$dt < 1.0} {
+	       set t $tmp[string trimleft [format "%.${ndec}f" \
+					       [expr {($t-int($t))}]] 0]
+	      } else {
+	       set t $tmp
+	      }
+	     }
+	     "PAL frames" {
+	      set t [expr {int($j * $dt * 25.0 + $ft)}]
+	     }
+	     "NTSC frames" {
+	      set t [expr {int($j * $dt * 30.0 + $ft)}]
+	     }
+	     "10ms frames" {
+	      set t [expr {int($j * $dt * 100.0 + $ft)}]
+	     }
 	    }
-
 	    if {$a(-draw0) == 1 || $j > 0 || $a(-starttime) > 0.0} {
 		$canvas create text [expr {$ox+$x}] [expr {$oy+$height}] \
 			-text $t -font $a(-font) -anchor s -tags $a(-tags) \
@@ -1096,6 +1167,8 @@ namespace eval snack {
 	    set lm(height) $a(-length)
 	    set lm(width)  $a(-width)
 	}
+	set lm(maxtime) [clock seconds]
+	set lm(maxlevel) 0.0
 
 	proc drawLevelMeter {w} {
             upvar [namespace current]::${w}::levelmeter lm
@@ -1107,18 +1180,24 @@ namespace eval snack {
 	    $c create rectangle 0 0 $lm(width) $lm(height) \
 		    -fill $lm(oncolor) -outline ""
 	    $c create rectangle 0 0 0 0 -outline "" -fill $lm(offcolor) \
-		    -tag mask
+		    -tag mask1
+	    $c create rectangle 0 0 0 0 -outline "" -fill $lm(offcolor) \
+		    -tag mask2
 	    $c create rectangle 0 0 [expr $lm(width)-1] [expr $lm(height)-1] \
 		    -outline $lm(bg)
 	    if {[string match horiz* $lm(orient)]} {
-		$c coords mask [expr {$lm(level)*$lm(width)}] 0 \
+		$c coords mask1 [expr {$lm(level)*$lm(width)}] 0 \
+			$lm(width) $lm(height)
+		$c coords mask2 [expr {$lm(level)*$lm(width)}] 0 \
 			$lm(width) $lm(height)
 		for {set x 5} {$x < $lm(width)} {incr x 5} {
 		    $c create line $x 0 $x [expr $lm(width)-1] -fill black \
 			    -width 2
 		}
 	    } else {
-		$c coords mask 0 0 $lm(width) \
+		$c coords mask1 0 0 $lm(width) \
+			[expr {$lm(height)-$lm(level)*$lm(height)}]
+		$c coords mask2 0 0 $lm(width) \
 			[expr {$lm(height)-$lm(level)*$lm(height)}]
 		for {set y 5} {$y < $lm(height)} {incr y 5} {
 		    $c create line 0 [expr $lm(height)-$y] \
@@ -1135,16 +1214,30 @@ namespace eval snack {
               switch -- [lindex $args 0] {
     	      -level {
 		  set arg [lindex $args 1]
-		  set arg [expr {$arg/32768.0}]
-		  if {$arg < 0.00001} { set arg 0.00001 }
-		  set lm(level) [expr {log($arg)/4.516+1.0}]
+   		  if {$arg < 1} { set arg 1 }
+		  set lm(level) [expr {log($arg)/10.3972}]
+		  if {[clock seconds] - $lm(maxtime) > 2} {
+		    set lm(maxtime) [clock seconds]
+		    set lm(maxlevel) 0.0
+		  }
+		  if {$lm(level) > $lm(maxlevel)} {
+		    set lm(maxlevel) $lm(level)
+		  }
+
 		  if {[string match horiz* $lm(orient)]} {
-		      ${w}_levelMeter coords mask \
-			      [expr {$lm(level)*$lm(width)}] 0 \
-			      $lm(width) $lm(height)
+		    set l1 [expr {5*int($lm(level)*$lm(width)/5)}]
+		    set l2 [expr {5*int($lm(maxlevel)*$lm(width)/5)}]
+		    ${w}_levelMeter coords mask1 $l2 0 \
+			$lm(width) $lm(height)
+		    ${w}_levelMeter coords mask2 [expr {$l2-5}] 0 \
+			$l1 $lm(height)
 		  } else {
-		      ${w}_levelMeter coords mask 0 0 $lm(width) \
-			      [expr {$lm(height)-$lm(level)*$lm(height)}]
+		    set l1 [expr {5*int($lm(level)*$lm(height)/5)}]
+		    set l2 [expr {5*int($lm(maxlevel)*$lm(height)/5)}]
+		    ${w}_levelMeter coords mask1 0 0 $lm(width) \
+			[expr {$lm(height)-$l2}]
+		    ${w}_levelMeter coords mask2 0 [expr {$lm(height)-$l2+5}] \
+			$lm(width) [expr {$lm(height)-$l1}]
 		  }	 
 	      }
 	      -length {
